

/*
=======================================
   Name 	Add 	    HADD
UART_RB_THR	000	Base + 0x00
UART_IER	001	Base + 0x04
UART_IIR_FCR	010	Base + 0x08
UART_LCR 	011	Base + 0x0C
UART_LSR	101	Base + 0x14
=======================================
*/

module board_fifo (User_mode_start, User_add, Enable, CLK, RSTn, LED, boardmem_addr_o, boardmem_dat_o_0, boardmem_dat_o_1, boardmem_dat_i_0, boardmem_dat_i_1, CE_o_0, CE_o_1, WE_o, LB_o_0, LB_o_1, UB_o_0, UB_o_1, OE_o, uart_addr_o, uart_wdata_o, uart_rdata_i, uart_we_o, uart_re_o);

parameter ST0 = 5'b00000;
parameter ST1 = 5'b00001;
parameter ST2 = 5'b00010;
parameter ST3 = 5'b00011;
parameter ST4 = 5'b00100;
parameter ST5 = 5'b00101;
parameter ST6 = 5'b00110;
parameter ST7 = 5'b00111;
parameter ST8 = 5'b01000;
parameter ST9 = 5'b01001;
parameter ST10 = 5'b01010; 
parameter ST11 = 5'b01011;
parameter ST12 = 5'b01100;
parameter ST13 = 5'b01101; // ST_FINISH
parameter ST14 = 5'b01110;
parameter ST15 = 5'b01111;
parameter ST16 = 5'b10000;
parameter ST17 = 5'b10001;
parameter ST18 = 5'b10010;
parameter ST19 = 5'b10011;
parameter ST20 = 5'b10100;
parameter ST21 = 5'b10101;
parameter ST22 = 5'b10110;
parameter ST23 = 5'b10111;
parameter ST24 = 5'b11000;
parameter ST25 = 5'b11001;
parameter ST26 = 5'b11010;

input User_mode_start;
input [1:0] User_add;
input Enable;

input CLK;
input RSTn;

output reg [7:0] LED;

output reg [17:0] boardmem_addr_o;

output reg [15:0] boardmem_dat_o_0;
output reg [15:0] boardmem_dat_o_1;

//inout wire [15:0] boardmem_dat_0;
//inout wire [15:0] boardmem_dat_1;

input wire [15:0] boardmem_dat_i_0;
input wire [15:0] boardmem_dat_i_1;

output reg CE_o_0; // // '0' is active  
output reg CE_o_1;

output reg WE_o; // '0' is write  and '1' is read 


output reg LB_o_0; // '0' is active
output reg LB_o_1; // '0' is active

output reg UB_o_0; // '0' is active
output reg UB_o_1; // '0' is active
output wire OE_o;

assign OE_o = ~WE_o;

output reg [2:0] uart_addr_o;
output reg [7:0] uart_wdata_o;
input wire [7:0] uart_rdata_i;
output reg uart_we_o;
output reg uart_re_o;

// separated output -by smkcow
/*
assign boardmem_dat_0 =   WE_o? 16'bz : boardmem_dat_o_0 ;  // WE_o is low active (writing is low active)
assign boardmem_dat_i_0 = WE_o?  boardmem_dat_0 : 16'bz ;  // WE_O value is 1 in case of  reading status

assign boardmem_dat_1 =   WE_o? 16'bz : boardmem_dat_o_1 ;  // WE_o is low active (writing is low active)
assign boardmem_dat_i_1 = WE_o?  boardmem_dat_1 : 16'bz ;  // WE_O value is 1 in case of  reading status
*/
wire internal_ram_EN;
wire internal_ram_WE;
wire [7:0] internal_ram_dat_o;
//wire [7:0] internal_ram_dat_i;
reg internal_ram_EN_reg;
//assign internal_ram_EN = 1'b1;
assign internal_ram_EN = internal_ram_EN_reg;

reg [4:0] state;
reg [2:0] cnt_relax; // time to between write finish and read starting
reg [1:0] cnt_during_value_assign; //
reg [8:0] cnt_total_data_depth; // address width calculating == 268 

reg [1:0] internal_ram_add;
reg internal_ram_WE_reg;

wire [1:0] real_Address_ram;
assign real_Address_ram = User_mode_start? User_add : internal_ram_add;
assign internal_ram_WE = internal_ram_WE_reg;


wire [3:0] transition_data_o;
reg flag_new;
reg [7:0] transition_data;
reg transition_enable;

reg_sram Ureg_sram(
	.Data(boardmem_dat_i_0[7:0]), 
	.Q(internal_ram_dat_o), 
	.Clock(CLK), 
	.WE(internal_ram_WE), 
	.Address({8'b00000000,real_Address_ram}), 
	.EN(internal_ram_EN)
);

transition Utransition(
	.transition_enable(transition_enable),
	.uart_rdata_i(uart_rdata_i),
	.transition_data_o(transition_data_o)
);


always @(negedge RSTn or posedge CLK)
begin
	if(RSTn == 1'b0)
	begin
		cnt_total_data_depth = 9'b100001011;
		cnt_during_value_assign <= 2'b00;

		//LED_OP <= 1'b0;
		//LED_DONE <= 1'b0;
		/*
			To board sram, initial value is for [not writing] == [read]
		*/
		boardmem_addr_o <= 18'b000000000000000000;

		boardmem_dat_o_0 <= 16'b0000000000000000;
		boardmem_dat_o_1 <= 16'b0000000000000000;
		internal_ram_WE_reg <= 1'b0;
	
		WE_o <= 1'b1;

		//cnt <= 18'b111111111111111111;
		cnt_relax <= 3'b111;

		state <= ST0;
		LED<= 8'h00;

		CE_o_0 <= 1'b1;
		CE_o_1 <= 1'b1;

		LB_o_0 <= 1'b1;
		UB_o_0 <= 1'b1;
		LB_o_1 <= 1'b1;
		UB_o_1 <= 1'b1;

		uart_addr_o <= 3'b000;
		uart_wdata_o <= 8'h00;
		uart_we_o <= 1'b0;
		uart_re_o <= 1'b0;
		transition_enable <= 1'b0;
		internal_ram_EN_reg <= 1'b0;

		flag_new <= 1'b0;
		transition_data <= 8'b00000000;
		
		internal_ram_add <= 2'b00;

	end
	else begin
		if (Enable == 1'b1)
		begin
			case (state) 
				/*
					From here, Uart initialization senario starts
				*/
       				ST0 : begin
					/*
						LCR = 0x83;   
					*/
					uart_addr_o <= 3'b011;
					uart_wdata_o <= 8'h83;
					uart_we_o <= 1'b1;
					uart_re_o <= 1'b0;

					state <= ST1;
				end	
				ST1 : begin
					/*
						IER = 0x00;
					*/
					//LED_OP <= 1'b1;
					uart_addr_o <= 3'b001;
					uart_wdata_o <= 8'h00;
					uart_we_o <= 1'b1;
					state <= ST2;
				end
				ST2 : begin
					/* 25MHz = 0xOE, 50MHz = 0x1B
						RB_THR = 0x0E
					*/
					uart_addr_o <= 3'b000;
					uart_wdata_o <= 8'h0E;
					uart_we_o <= 1'b1;
					state <= ST3;
				end
				ST3 : begin
					/*
						LCR = 0x03;   
					*/
					uart_addr_o <= 3'b011;
					uart_wdata_o <= 8'h03;
					uart_we_o <= 1'b1;
					state <= ST4;

				end
				ST4 : begin
					/*
						IIR_FCR = 0x01;   
					*/
					uart_addr_o <= 3'b010;
					uart_wdata_o <= 8'h01;
					uart_we_o <= 1'b1;
					state <= ST5;
				end
				ST5 : begin
					/*
						IER = 0x01;   
					*/
					uart_addr_o <= 3'b001;
					uart_wdata_o <= 8'h01;
					uart_we_o <= 1'b1;
					state <= ST6;
				end
				ST6: begin
					// test pattern == just write 'T' character to the terminal screen

					// below 4 lines is disables in case with tty test
					uart_addr_o <= 3'b000;
					uart_we_o <= 1'b1;
					uart_re_o <= 1'b0;
					uart_wdata_o <= 8'h54;
					state <= ST7;
				end				
				ST7 : begin
					/*
						Iteration from this state to ST12
						1_1 - READ from LSR :  Don't need to wait 1 or 2 cycle - immediately reading 
						1_2 - check LSR value is 0x21
						2 - Then, Read from Buffer(RB_THR) and write memory
					*/
					uart_addr_o <= 3'b101;
					uart_we_o <= 1'b0;
					uart_re_o <= 1'b1;
					state <= ST8;
				end

					/*
						LSR value == 0x21 ?? Receive at least one character?
					*/
				ST8 : begin	
					if(uart_rdata_i[0]) begin
					/*	cnt_1 <= cnt_1 -1'b1;
						if(cnt_1 ==8'h00) begin
							*/
						uart_addr_o <= 3'b000;
						uart_we_o <= 1'b0;
						uart_re_o <= 1'b1;
						state <= ST9;
					end
					else
						state <= ST7;

				end
					/*
						READ from RB_THR;   
					*/
				ST9 : begin  // determine if received character is space or \n in a hex file
					uart_re_o <= 1'b0;
					if(uart_rdata_i==8'h20 || uart_rdata_i ==8'h0a) begin
							transition_enable <= 1'b0;
							state <= ST7;
					end
					else begin
							transition_enable <= 1'b1;
							state <= ST10;						
					end
				end
				ST10 : begin // make 8 bits using 4 bits data so it needs doing twice and endian processing
					if(!flag_new) begin
							transition_data[7:4] <= transition_data_o;
							flag_new <= ~flag_new;
							state <= ST7;
					end
					else begin
							transition_data[3:0] <= transition_data_o;
							flag_new <= ~flag_new;
							WE_o <= 1'b0;
							state <= ST11;
					end

				end
				ST11 : begin
					/*
					Write to board sram   
					- wait one cycle because the write cycle of memory needs control signals first	
					- board address has given at the reset status
					- WE signal has given at the prior status at ST10
					  bacause a board mem needs the control signal prior to give address and data value 
					*/
					case(cnt_during_value_assign)
						2'b00 : begin
							CE_o_0 <= 1'b1;
							CE_o_1 <= 1'b0;

							LB_o_0 <= 1'b1;
							UB_o_0 <= 1'b1;

							LB_o_1 <= 1'b1;
							UB_o_1 <= 1'b0;

							cnt_during_value_assign <= cnt_during_value_assign + 1'b1;
							state <= ST7;
						end
						2'b01 : begin
							CE_o_0 <= 1'b1;
							CE_o_1 <= 1'b0;

							LB_o_0 <= 1'b1;
							UB_o_0 <= 1'b1;

							LB_o_1 <= 1'b0;
							UB_o_1 <= 1'b1;

							cnt_during_value_assign <= cnt_during_value_assign + 1'b1;
							state <= ST7;
						end
						2'b10 : begin
							CE_o_0 <= 1'b0;
							CE_o_1 <= 1'b1;

							LB_o_0 <= 1'b1;
							UB_o_0 <= 1'b0;

							LB_o_1 <= 1'b1;
							UB_o_1 <= 1'b1;							

							cnt_during_value_assign <= cnt_during_value_assign + 1'b1;
							state <= ST7;
						end
						2'b11 : begin
							CE_o_0 <= 1'b0;
							CE_o_1 <= 1'b1;

							LB_o_0 <= 1'b0;
							UB_o_0 <= 1'b1;

							LB_o_1 <= 1'b1;
							UB_o_1 <= 1'b1;

							cnt_during_value_assign <= cnt_during_value_assign + 1'b1;
							state <= ST12;
						end



					endcase

					boardmem_dat_o_0 <= {transition_data,transition_data};
					boardmem_dat_o_1 <= {transition_data,transition_data};
					WE_o <= 1'b0;

				end
				ST12 : begin
					/*
						prepare to write again   
						1 - increase boardmem_addr
						2 - decrease internal counter value to 0 
					*/
				       // Total writing was not finished so WE value doesn't need to be '0' 							
				       boardmem_addr_o <= boardmem_addr_o + 1'b1;
				       cnt_total_data_depth = cnt_total_data_depth - 1'b1;
				       if(cnt_total_data_depth == 9'b000000000) begin
						uart_re_o <= 1'b0;
						state <= ST13;
						end
				       else
						state <= ST7;
				end
				

			       // *****  ST_FINISH is ST13 **** 
				ST13 : begin
					LED <= 8'b11111111;
					state <= ST14;
				end
				ST14: begin
					state <= ST15;
				end
				ST15 : begin
					state <= ST16;
				end
				// Go to the Debugging mode ..... 
				// Read from the external memory and write to
				// the internal memory 
				// After that, read from the internal memory
				// and write to LED ... so user can see the
				// value is right or not
				ST16 : begin
					state <= ST17;
					WE_o <= 1'b1;  // prior action is needed == it's very important
				end
				ST17 : begin
					transition_enable <= 1'b0;
					internal_ram_EN_reg <= 1'b1;
					// writing was finished so WE value will be '1' ==> WE_o is low active 				
					boardmem_addr_o <= 18'b000000000000001100;
					WE_o <= 1'b1;  // 1 value is for reading of external memory

					internal_ram_add <= 2'b00;
					internal_ram_WE_reg <= 1'b1; // 1 value is for writing of internal memory
					// mem_dat_i is automatically assigned to internal ram
					
					CE_o_0 <= 1'b0;
					CE_o_1 <= 1'b1;

					LB_o_0 <= 1'b0;
					UB_o_0 <= 1'b1;

					LB_o_1 <= 1'b1;
					UB_o_1 <= 1'b1;

					state <= ST18;
				end
				ST18 : begin
					state <= ST19;
				end
				ST19: begin
					boardmem_addr_o <= boardmem_addr_o+1'b1;
					WE_o <= 1'b1;

					internal_ram_add <= 2'b01;
					internal_ram_WE_reg <= 1'b1;
					// mem_dat_i is automatically assigned to internal ram

					state <= ST20;
				end
				ST20: begin
					state <= ST21;
				end
				ST21 : begin
					boardmem_addr_o <= boardmem_addr_o+1'b1;
					WE_o <= 1'b1;

					internal_ram_add <= 2'b10;
					internal_ram_WE_reg <= 1'b1;
					// mem_dat_i is automatically assigned to internal ram

					state <= ST22;
				end
				ST22 : begin
					state <= ST23;
				end
				ST23 : begin
					boardmem_addr_o <= boardmem_addr_o+1'b1;
					WE_o <= 1'b1;

					internal_ram_add <= 2'b11;
					internal_ram_WE_reg <= 1'b1;
					// mem_dat_i is automatically assigned to internal ram

					state <= ST24;
				end
				ST24 : begin
					state <= ST25;
				end
				ST25 : begin
					internal_ram_WE_reg <= 1'b0;
					if(User_mode_start == 1'b1)
						state <= ST26;
					else
						state <= ST25;	
				end
				ST26 : begin
					internal_ram_WE_reg <= 1'b0;
					state <= ST26;
					if(User_mode_start == 1'b1)
						LED <= internal_ram_dat_o;
					else 
						LED <= 8'h00;
				end
				default: begin
					state <= ST0;
				end
			endcase	
	end
	else begin
		
		
			/*
				To board sram, initial value is for [not writing] == [read]
			*/
			transition_enable <= 1'b0;

			cnt_total_data_depth = 9'b100001011;
			cnt_during_value_assign <= 2'b00;

			boardmem_addr_o <= 18'b000000000000000000;

			boardmem_dat_o_0 <= 16'b0000000000000000;
			boardmem_dat_o_1 <= 16'b0000000000000000;
	
			WE_o <= 1'b1;

			cnt_relax <= 3'b111;
			state <= ST0;
			LED<= 8'h00;
			
			CE_o_0 <= 1'b1;
			CE_o_1 <= 1'b1;

			LB_o_0 <= 1'b1;
			UB_o_0 <= 1'b1;
			LB_o_1 <= 1'b1;
			UB_o_1 <= 1'b1;
			internal_ram_WE_reg <= 1'b0;
			internal_ram_EN_reg <= 1'b0;
			internal_ram_add <= 2'b00;

			uart_addr_o <= 3'b000;
			uart_wdata_o <= 8'h00;
			uart_we_o <= 1'b0;
			uart_re_o <= 1'b0;
			flag_new <= 1'b0;
			transition_data <= 8'b00000000;

		end 
	end
end








endmodule   
  
