// IS61LV25616 Asynchronous SRAM, 256K x 16 = 4M; speed: 10ns.
// Note; 1) Please include "+define+ OEb" in running script if you want to check
//          timing in the case of OE_ being set.
//       2) Please specify access time by defining tAC_10 or tAC_12.

// `define OEb
`define tAC_10
`timescale 1ns/10ps


module IS61LV25616 (A, IO_0, IO_1, CE_0, CE_1, OE_0, OE_1, WE_0, WE_1, LB_0, LB_1, UB_0, UB_1);

//parameter FILE = "C:/CoreIDEA/examples/EX_0001_amba_platform/sw/uart/uart.hex";
//parameter FILE = "C:/CoreIDEA/inspire_ver_31_core_a-exams/EX_0004_TLCD/sw/uart_tlcd/uart_tlcd_test.hex";
//parameter FILE = "D:/test/fifo/mem_fifo/reference_design_TLCD/EX_0004_TLCD/sw/uart.hex";

parameter dqbits = 32;
parameter memdepth = 262144;
parameter addbits = 18;
/*
parameter dqbits = 16;
parameter memdepth = 262143;
parameter addbits = 18;
*/
parameter Toha  = 2;

parameter Tsa   = 2;
`define tAC_10

`ifdef tAC_10
  parameter Taa   = 10,
            Thzce = 3,
	    Thzwe = 5;
`endif

`ifdef tAC_12
  parameter Taa   = 12,
	    Thzce = 5,
	    Thzwe = 6;
`endif

input CE_0, OE_0, WE_0, LB_0, UB_0;
input CE_1, OE_1, WE_1, LB_1, UB_1;

input [(addbits - 1) : 0] A;

inout [15 : 0] IO_0;
inout [15 : 0] IO_1;
 
wire [15 : 0] dout_0;
wire [15 : 0] dout_1;
reg  [31 : 0] bank0 [0 : memdepth-1];

// wire [(dqbits - 1) : 0] memprobe = {bank1[A], bank0[A]};

wire r_en_0 = WE_0 & (~CE_0) & (~OE_0);
wire r_en_1 = WE_1 & (~CE_1) & (~OE_1);

wire w_en_0 = (~WE_0) & (~CE_0) & ((~LB_0) | (~UB_0));
wire w_en_1 = (~WE_1) & (~CE_1) & ((~LB_1) | (~UB_1));

assign #(r_en_0 ? Taa : Thzce) IO_0 = r_en_0 ? dout_0 : 16'bz;   
assign #(r_en_1 ? Taa : Thzce) IO_1 = r_en_1 ? dout_1 : 16'bz;   

initial
begin
//	$readmemh(FILE, bank0);
	$timeformat (-9, 0.1, " ns", 10);
end

assign dout_0[7:0] = LB_0 ? 8'bz : bank0[A][7:0];
assign dout_0[15:8] = UB_0 ? 8'bz : bank0[A][15:8];

assign dout_1[7:0] = LB_1 ? 8'bz : bank0[A][23:16];
assign dout_1[15:8] = UB_1 ? 8'bz : bank0[A][31:24];

always @(A or w_en_0 or w_en_1)
  begin
    #Tsa
    if (w_en_0)
      #Thzwe
      begin
	bank0[A][7:0] = LB_0 ? bank0[A][7:0] : IO_0 [7 : 0];
	bank0[A][15:8] = UB_0 ? bank0[A][15:8] : IO_0 [15: 8];
      end
      if (w_en_1)
      #Thzwe
      begin
	bank0[A][23:16] = LB_1 ? bank0[A][23:16] : IO_1 [7 : 0];
	bank0[A][31:24] = UB_1 ? bank0[A][31:24] : IO_1 [15: 8];
      end
  end
 
// Timing Check
`ifdef tAC_10
  specify
    specparam
      tSA   = 0,
      tAW   = 8,
      tSCE  = 8,
      tSD   = 6,
      tPWE2 = 10,
      tPWE1 = 8,
      tPBW  = 8;
`else

`ifdef tAC_10
  specify
    specparam
      tSA   = 0,
      tAW   = 8,
      tSCE  = 8,
      tSD   = 6,
      tPWE2 = 12,
      tPWE1 = 8,
      tPBW  = 8;
`endif
`endif

    $setup (A, negedge CE_0, tSA);
        $setup (A, negedge CE_1, tSA);
//     $setup (A, posedge CE_, tAW);
//     $setup (IO, posedge CE_, tSD);
    $setup (A, negedge WE_0, tSA);
        $setup (A, negedge WE_1, tSA);
//     $setup (IO, posedge WE_, tSD);
    $setup (A, negedge LB_0, tSA);
        $setup (A, negedge LB_1, tSA);
	$setup (A, negedge UB_0, tSA);
	$setup (A, negedge UB_1, tSA);

    $width (negedge CE_0, tSCE);
    $width (negedge CE_1, tSCE);

    $width (negedge LB_0, tPBW);
    $width (negedge LB_1, tPBW);

    $width (negedge UB_0, tPBW);
    $width (negedge UB_1, tPBW);
    `ifdef OEb
    $width (negedge WE_0, tPWE1);
    $width (negedge WE_1, tPWE1);
    `else
    $width (negedge WE_0, tPWE2);
    $width (negedge WE_1, tPWE2);
    `endif 

  endspecify

endmodule

