`timescale 1ns/1ps

module TB_top_uart_fifo();
defparam Utty._UART_CLOCK_ = 25000000;
defparam Utty._BAUD_RATE_ = 115200;

reg CLOCK;
reg RESETn;

 
reg  Enable;
reg [1:0] User_add; //
reg User_mode_start; // 
wire  [7:0] LED;

wire  [17:0] A;
wire  [15:0] IO_0;
wire  [15:0] IO_1;
wire  CE_0;
wire  CE_1;

wire  WE;
wire  OE;

wire  LB_0;
wire  LB_1;

wire  UB_0;
wire  UB_1;

wire  UART_TX1;
wire UART_RX1;
//reg  UART_RX1;

top_uart_fifo Utop_uart_fifo(
	.Enable(Enable),
	.User_add(User_add),
	.User_mode_start(User_mode_start),
//	.Mem_Sel(Mem_Sel),
   	.LED(LED),
	.A(A),
	.IO_0(IO_0),
	.IO_1(IO_1),
	
	.CE_0(CE_0),
	.CE_1(CE_1),
	
	.WE(WE),
	.OE(OE),
	
	.LB_0(LB_0),
	.LB_1(LB_1),
	
	.UB_0(UB_0),
	.UB_1(UB_1),

	.RESETn(RESETn),        // active-low power-up reset == ifp_resetb
	.CLOCK(CLOCK),         // system clock == ifp_clk == iwp_clk
	.UART_TX1(UART_TX1),
	.UART_RX1(UART_RX1)
);


IS61LV25616 U_IS61LV25616(
	.A(A),
       	.IO_0(IO_0), 
	.IO_1(IO_1), 
	.CE_0(CE_0), 
	.CE_1(CE_1), 
	.OE_0(~WE), 
	.OE_1(~WE), 
	.WE_0(WE), 
	.WE_1(WE), 
	.LB_0(LB_0), 
	.LB_1(LB_1), 
	.UB_0(UB_0), 
	.UB_1(UB_1)
);

tty Utty(
       .RESETn(RESETn),
       .CLK(CLOCK),
       .STX(UART_RX1),
       //.SRX(UART_TX1), // this lines is disables in case with tty test because of T character
       .Enable(Enable)
);


	
initial begin


	CLOCK = 1'b0;
	Enable = 1'b1;
	RESETn = 1'b0;
	User_mode_start = 1'b0;
	User_add = 2'b00;

	#100 RESETn = 1'b1;

	#200000000 User_add = 2'b00;
	
	#20 User_mode_start = 1'b1; 	
	#200	User_add = 2'b01;
	#200	User_add = 2'b10;
	#200	User_add = 2'b11;

	#1000 Enable = 1'b0;
	#100 	RESETn = 1'b0;
	#50	RESETn = 1'b1;
end

always begin
	#20 CLOCK = ~CLOCK;
end

endmodule 
