//----------------------------------------------------------------
//  Copyright (c) by Dynalith Systems Co., Ltd.
//  All right reserved.
//  http://www.dynalith.com
//----------------------------------------------------------------
// tty.v
//----------------------------------------------------------------
// VERSION: 2009.01.14.
//----------------------------------------------------------------

`timescale 1ns/10ps

module tty (
       RESETn,
       CLK,
       STX,
       SRX,
       Enable
);

parameter FILE = "C:/CoreIDEA/examples/EX_0001_amba_platform/sw/uart/uart.hex";
parameter memdepth = 262144;
parameter _UART_CLOCK_ = 16500000,
          _BAUD_RATE_  = 19200;

   parameter ST0 = 5'b00000;
   parameter ST1 = 5'b00001;
   parameter ST2 = 5'b00010;
   parameter ST3 = 5'b00011;
   parameter ST4 = 5'b00100;
   parameter ST5 = 5'b00101;
   parameter ST6 = 5'b00110;
   parameter ST7 = 5'b00111;
   parameter ST8 = 5'b01000;
   parameter ST9 = 5'b01001;
   parameter ST10 =5'b01010;
   parameter ST11= 5'b01011;
   parameter ST12= 5'b01100;
   parameter ST13= 5'b01101;
   parameter ST14= 5'b01110;
   parameter ST15= 5'b01111;
   parameter ST16= 5'b10000;
   parameter ST17= 5'b10001;
   parameter ST18= 5'b10010;
   parameter ST19= 5'b10011;
   parameter ST20= 5'b10100;
   parameter ST21= 5'b10101;
   parameter ST22= 5'b10110;
   parameter ST23= 5'b10111;
   parameter ST24= 5'b11000;
   parameter ST25= 5'b11001;
   parameter ST26= 5'b11010;
   parameter ST27= 5'b11011;
   parameter ST28= 5'b11100;
   parameter ST29= 5'b11101;
   parameter ST30= 5'b11110;



   input  RESETn; wire        RESETn;
   input  CLK;    wire        CLK;
   output STX;    wire        STX;
   input  SRX;    wire        SRX;
   input  Enable;    wire     Enable;
   //--------------------------------------------
   wire       interrupt;
   reg        reg_we;   // Write enable for registers
   reg        reg_re;   // Read enable for registers
   reg  [2:0] reg_adr;
   reg  [7:0] reg_dat8_i;
   wire [7:0] reg_dat8_o;
   //--------------------------------------------
   wire   ctsn = 1'b0;
   wire   dtr_pad_o;
   wire   dsr_pad_i=1'b0;
   wire   ri_pad_i =1'b0;
   wire   dcd_pad_i=1'b0;
   wire   rts_internal;
   //---------------------------
   // U16550 CSR address
   localparam RB_THR     =0;
   localparam IER        =1;
   localparam IIR_FCR    =2;
   localparam LCR        =3;
   localparam MCR        =4;
   localparam LSR        =5;
   localparam MSR        =6;
   //--------------------------------------------
   reg [4:0] state;
   reg transition_enable;
   integer i;
 
   reg  [31 : 0] bank0 [0 : memdepth-1];
   reg [3:0] char;
   reg [17:0] A;
   wire [7:0] transition_data_o;
   wire [7:0] reg_dat8_i_wire;
   
   

reg start;
reg [15:0] doing;
wire done;




assign reg_dat8_i_wire = reg_dat8_i;

   uart_regs Uregs(
          .clk         (CLK),
          .wb_rst_i    (~RESETn),
          .wb_addr_i   (reg_adr),
          .wb_dat_i    (reg_dat8_i_wire),
          .wb_dat_o    (reg_dat8_o),
          .wb_we_i     (reg_we),
          .wb_re_i     (reg_re),
          .modem_inputs({~ctsn, dsr_pad_i, ri_pad_i,  dcd_pad_i}),
          .stx_pad_o   (STX),
          .srx_pad_i   (SRX),
          .rts_pad_o   (rts_internal),
          .dtr_pad_o   (dtr_pad_o),
          .int_o       (interrupt)
   );
   //--------------------------------------------
transition_tty Utransition_tty(
	.transition_enable(transition_enable),
	.input_data(char), 
	.transition_data_o(transition_data_o)
);

	initial
	begin
		$readmemh(FILE, bank0);
		$timeformat (-9, 0.1, " ns", 10);
	end


assign done = doing[15];

always @(negedge RESETn or posedge CLK)
begin
	if(RESETn == 1'b0) begin
		doing <= 16'h0000;
	end else if(start == 1'b1)
		doing <= doing + 1'b1;
end

always @(negedge RESETn or posedge CLK)
begin
	if(RESETn == 1'b0)
	begin
		state <= ST0;
		transition_enable <= 1'b0;
		char <= 4'b0000; 
		A <= 18'b00_0000_0000_0000_0000;
		reg_we <= 1'b0;   // Write enable for registers
		reg_re <= 1'b0;   // Read enable for registers
		reg_adr <= 3'b000;
		reg_dat8_i <= 8'h00;
		start <= 1'b0;
	end
	else begin
		if (Enable == 1'b1)
		begin
		case (state) 
				/*
					From here, Uart initialization senario starts
				*/
       				ST0 : begin
					/*
						LCR = 0x83;   
					*/
					reg_adr <= 3'b011;
					reg_dat8_i <= 8'h83;
					reg_we <= 1'b1;
					reg_re <= 1'b0;

					state <= ST1;


				end	
				ST1 : begin
					/*
						IER = 0x00;
					*/
					//LED_OP <= 1'b1;
					reg_adr <= 3'b001;
					reg_dat8_i <= 8'h00;
					reg_we <= 1'b1;
					state <= ST2;
				end
				ST2 : begin
					/* 25MHz = 0xOE, 50MHz = 0x1B
						RB_THR = 0x0E
					*/
					reg_adr <= 3'b000;
					reg_dat8_i <= 8'h0E;
					reg_we <= 1'b1;
					state <= ST3;
				end
				ST3 : begin
					/*
						LCR = 0x03;   
					*/
					reg_adr <= 3'b011;
					reg_dat8_i <= 8'h03;
					reg_we <= 1'b1;
					state <= ST4;

				end
				ST4 : begin
					/*
						IIR_FCR = 0x01;   
					*/
					reg_adr <= 3'b010;
					reg_dat8_i <= 8'h01;
					reg_we <= 1'b1;
					state <= ST5;
				end
				ST5 : begin
					/*
						IER = 0x01;   
					*/
					reg_adr <= 3'b001;
					reg_dat8_i <= 8'h01;
					reg_we <= 1'b1;
					state <= ST6;
				end
				ST6 : begin
					start <= 1'b1;
					if(done == 1'b1) begin
						state <= ST7;
						transition_enable <= 1'b1;
						char <= bank0[A][31:28];
					end					
					else state <= ST6;
				end
				ST7 : begin
					// test pattern == just write 'T' character to the terminal screen
					// below 4 lines is disables in case with tty test
					//
					reg_adr <= 3'b000;
					reg_we <= 1'b1;
					reg_re <= 1'b0;
					//transition_enable <= 1'b1;
					//char <= bank0[A][31:28];
					reg_dat8_i <= transition_data_o;
					state <= ST8;
				end				
				ST8 : begin
					reg_adr <= 3'b101;
					reg_we <= 1'b0;
					reg_re <= 1'b1;
					state <= ST9;
				end
				ST9: begin
					if(reg_dat8_o[5] == 1'b1) begin
						state <= ST10;
						transition_enable <= 1'b1;
						char <= bank0[A][27:24];
					end else begin
						transition_enable <= 1'b0;
						state <= ST9;
					end
				end
				ST10 : begin
					reg_adr <= 3'b000;
					reg_we <= 1'b1;
					reg_re <= 1'b0;
					reg_dat8_i <= transition_data_o;
					state <= ST11;
				end
				ST11: begin
					reg_adr <= 3'b101;
					reg_we <= 1'b0;
					reg_re <= 1'b1;
					state <= ST12;
				end
				ST12 : begin
					if(reg_dat8_o[5] == 1'b1) begin				
						transition_enable <= 1'b1;
						char <= bank0[A][23:20];
						state <= ST13;
					end else begin
						transition_enable <= 1'b0;
						state <= ST12;
					end
				end

				ST13 : begin
					reg_adr <= 3'b000;
					reg_we <= 1'b1;
					reg_re <= 1'b0;					
					reg_dat8_i <= transition_data_o;
					state <= ST14;
				end
				ST14: begin					
					reg_adr <= 3'b101;
					reg_we <= 1'b0;
					reg_re <= 1'b1;
					state <= ST15;
				end
				ST15 : begin
					if(reg_dat8_o[5] == 1'b1) begin
						transition_enable <= 1'b1;
						char <= bank0[A][19:16];
						state <= ST16;
					end else begin
						transition_enable <= 1'b0;
						state <= ST15;
					end
				end

				ST16 : begin
					reg_adr <= 3'b000;
					reg_we <= 1'b1;
					reg_re <= 1'b0;					
					reg_dat8_i <= transition_data_o;
					state <= ST17;
				end
				ST17: begin					
					reg_adr <= 3'b101;
					reg_we <= 1'b0;
					reg_re <= 1'b1;
					state <= ST18;
				end
				ST18 : begin
					if(reg_dat8_o[5] == 1'b1) begin
						transition_enable <= 1'b1;
						char <= bank0[A][15:12];		
						state <= ST19;
					end else begin
						transition_enable <= 1'b0;
						state <= ST18;
					end
				end

				ST19 : begin
					reg_adr <= 3'b000;
					reg_we <= 1'b1;
					reg_re <= 1'b0;					
					reg_dat8_i <= transition_data_o;
					state <= ST20;
				end
				ST20: begin					
					reg_adr <= 3'b101;
					reg_we <= 1'b0;
					reg_re <= 1'b1;
					state <= ST21;
				end
				ST21 : begin
					if(reg_dat8_o[5] == 1'b1) begin
						transition_enable <= 1'b1;
						char <= bank0[A][11:8];	
						state <= ST22;
					end else begin
						transition_enable <= 1'b0;
						state <= ST21;
					end
				end

				ST22 : begin
					reg_adr <= 3'b000;
					reg_we <= 1'b1;
					reg_re <= 1'b0;					
					reg_dat8_i <= transition_data_o;
					state <= ST23;
				end
				ST23: begin					
					reg_adr <= 3'b101;
					reg_we <= 1'b0;
					reg_re <= 1'b1;
					state <= ST24;
				end
				ST24 : begin
					if(reg_dat8_o[5] == 1'b1) begin
						transition_enable <= 1'b1;
						char <= bank0[A][7:4];
						state <= ST25;
					end else begin
						transition_enable <= 1'b0;
						state <= ST24;
					end
				end

				ST25 : begin
					reg_adr <= 3'b000;
					reg_we <= 1'b1;
					reg_re <= 1'b0;					
					reg_dat8_i <= transition_data_o;
					state <= ST26;
				end
				ST26: begin					
					reg_adr <= 3'b101;
					reg_we <= 1'b0;
					reg_re <= 1'b1;
					state <= ST27;
				end
				ST27 : begin
					if(reg_dat8_o[5] == 1'b1) begin
						transition_enable <= 1'b1;
						char <= bank0[A][3:0];
						state <= ST28;
					end else begin
						transition_enable <= 1'b0;
						state <= ST27;
					end
				end

				ST28 : begin
					reg_adr <= 3'b000;
					reg_we <= 1'b1;
					reg_re <= 1'b0;					
					reg_dat8_i <= transition_data_o;
					state <= ST29;
				end
				ST29: begin					
					reg_adr <= 3'b101;
					reg_we <= 1'b0;
					reg_re <= 1'b1;
					A<= A+1'b1;
					state <= ST30;
				end
				ST30 : begin
					if(reg_dat8_o[5] == 1'b1) begin	
						state <= ST7;
						transition_enable <= 1'b1;
						char <= bank0[A][31:28];
					end else begin
						transition_enable <= 1'b0;
						state <= ST30;
					end
				end
			endcase
		end 
		else begin
			state <= 5'bzzzzz;
			transition_enable <= 1'bz;
			char <= 4'bzzzz; 
			A <= 18'bzz_zzzz_zzzz_zzzz_zzzz;
			reg_we <= 1'bz;   // Write enable for registers
			reg_re <= 1'bz;   // Read enable for registers
			reg_adr <= 3'bzzz;
			reg_dat8_i <= 8'hzz;
			start <= 1'bz;
	end
		end
	end
	endmodule


